/******************************************************************************
 File:    cpc.h
 Purpose: Header file for sample C language control programs for Hostess i/S2.
 Company: Comtrol Corporation
 Author:  Craig Harrison
 Ident:   $Id$
*******************************************************************************
 Copyright 1994 Comtrol Corporation. All rights reserved.
 Subject to developers license agreement.
******************************************************************************/

#ifndef DPRAM_H
#include "dpram.h"
#endif


// ----- Communication mode enables -------------------------------------------
// To enable compilation of the code for a communication mode for a line, set
// it's #define to 1, to disable set it to 0.  Don't enable more than one
// mode per line.
#define LN0_ASYNC  1           // set to 1 for line 0 asynchronous
#define LN1_ASYNC  1           // set to 1 for line 1 asynchronous
#define LN0_HDLC   0           // set to 1 for line 0 HDLC
#define LN1_HDLC   0           // set to 1 for line 1 HDLC
#define LN0_BISYNC 0           // set to 1 for line 0 Bisync
#define LN1_BISYNC 0           // set to 1 for line 1 Bisync


// ----- Check for invalid communication mode enables --------------------------
#if (LN0_ASYNC && LN0_HDLC) || (LN1_ASYNC && LN1_HDLC) || \
    (LN0_ASYNC && LN0_BISYNC) || (LN1_ASYNC && LN1_BISYNC) || \
    (LN0_HDLC && LN0_BISYNC) || (LN1_HDLC && LN1_BISYNC)
#error Invalid communication mode enables
#endif


// ----- Special test modes ---------------------------------------------------
// Set all of these to 0 for normal control program operation.  If enabling a
// test mode set only one of these to 1.
#define TEST_ASYNC 0           // set to 1 for line 0 test async mode
#define TEST_SFG1  0           // set to 1 to run TestSfg1()
#define TEST_SFG2  0           // set to 1 to run TestSfg2()
#define TEST_HDLC  0           // set to 1 for line 0, 1 test HDLC mode
#define T_DATARATE 64000L      // data rate for TEST_HDLC
#define T_TXLEN 64             // Tx frame length for TEST_HDLC


// ----- Modem change SYS message enable --------------------------------------
// Set this if you want modem input changes to be reported to the system.
#define TEST_MDMCHGMSG 0


// ----- Interrupt Controller (PIC) Registers -----------------------------
#define EOIVAL    0x20         // end of interrupt value
#define DMATC_TYPE 0x30        // DMA Terminal Count interrupt vector type
#define SYS_TYPE  0x33         // system interrupt vector type
#define INTCTL    0x9070       // PIC port for EOI
#define INTCTL1   0x9071       // PIC port for mask register


// ----- Timer Registers --------------------------------------------------
#define CNT100       0x1e84    // 100/sec counter value
#define CNT200       0x0f42    // 200/sec counter value
#define TIM0_TYPE    0x34      // timer 0 interrupt vector type
#define TIM1_TYPE    0x35      // timer 1 interrupt vector type
#define TIM2_TYPE    0x36      // timer 2 interrupt vector type
#define TIMER0       0x9074    // timer 0 count reg address
#define TIMER1       0x9075    // timer 1 count reg address
#define TIMER2       0x9076    // timer 2 count reg address
#define TIM_CTL      0x9077    // timer control word address
#define TIM0_CTLWD   0x64      // timer 0 control word
#define TIM1_CTLWD   0x74      // timer 1 control word
#define TIM2_CTLWD   0xb4      // timer 2 control word
#define MIN_TIMER    0         // minimum timer number
#define MAX_TIMER    2         // maximum timer number

typedef struct                   // timer information
{
   WORD_T VectorType;            // interrupt vector type number
   WORD_T ControlWord;           // counter control word
   WORD_T CountReg;              // counter register address
} TIMER_T;


// ----- V53 DMA controller -----------------------------------------------
#define DICM        0x9060             // DMA controller register addresses
#define DCH         0x9061
#define DCC         0x9062
#define DCA         0x9064
#define DCA_U       0x9066
#define DDC         0x9068
#define DMD         0x906a
#define DST         0x906b
#define DMK         0x906f

#define DDMA        0x04               // Disable DMA bit of DDC low byte

#define TC0         0x01               // DST register values
#define TC1         0x02
#define TC2         0x04
#define TC3         0x08

#define LN0_DISTXDMA 0x01             // DMK register values
#define LN0_DISRXDMA 0x02
#define LN1_DISTXDMA 0x04
#define LN1_DISRXDMA 0x08
#define LN0_ENTXDMA  0x0e
#define LN0_ENRXDMA  0x0d
#define LN1_ENTXDMA  0x0b
#define LN1_ENRXDMA  0x07
#define DMAMASK      0x0f

#define LN0_TXDMACH 0                  // line 0 Tx DMA channel
#define LN1_TXDMACH 2                  // line 0 Rx DMA channel
#define LN0_RXDMACH 1                  // line 1 Tx DMA channel
#define LN1_RXDMACH 3                  // line 1 Rx DMA channel


// ----- Configuration Control Register ---------------------------------------
#define CFGCTL_READINT  0x24     // read register software int
#define CFGCTL_WRITEINT 0x25     // write register software int
#define CFGCTL_IO       0xfe5e   // I/O address
#define LN0_TXCLK_DTE   0x0001   // line 0 Tx clock DTE source
#define LN0_ALTDTR_EN   0x0002   // line 0 alternate DTR enable
#define LN0_ALTDTR      0x0004   // line 0 alternate DTR
#define LN1_TXCLK_DTE   0x0010   // line 1 Tx clock DTE source
#define LN1_ALTDTR_EN   0x0020   // line 1 alternate DTR enable
#define LN1_ALTDTR      0x0040   // line 1 alternate DTR
#define EPROM_DIS       0x0100   // EPROM disable


// ----- SCC command register I/O addresses -------------------------------
#define LN00_IO         0xe1f4
#define LN01_IO         0xe1f0


// ----- SCC register defines ---------------------------------------------
#define WR0      0
#define WR1      1
#define WR2      2
#define WR3      3
#define WR4      4
#define WR5      5
#define WR6      6
#define WR7      7
#define WR8      8
#define WR9      9
#define WR10     10
#define WR11     11
#define WR12     12
#define WR13     13
#define WR14     14
#define WR15     15

#define RR0      0
#define RR1      1
#define RR2      2
#define RR3      3
#define RR8      8
#define RR10     10
#define RR12     12
#define RR13     13
#define RR15     15


// ----- Baud Rate Generator Time Constants - x16 Baud Rate Factor -------
// These used for asynchronous baud rates.
// Based on a 7.3728 MHz clock.  The formula for computing these is:
// BRG_TC = (Clock / (2 * Baud_Rate_Factor * Baud_Rate)) - 2
//        = (7372800 / (2 * 16 * Baud_Rate)) - 2
//        = (230400 / Baud_Rate) - 2
// Refer to Am85C30 SCC 1992 Technical Manual, section 7.2.3.1 - SCC
// Operating Mode Programming
#define BPS50           4606
#define BPS75           3070
#define BPS110          2093           // 0.022 percent error
#define BPS134          1717           // 0.023 percent error
#define BPS150          1534
#define BPS300          766
#define BPS600          382
#define BPS1200         190
#define BPS1800         126
#define BPS2000         113            // 0.177 percent error
#define BPS2400         94
#define BPS3600         62
#define BPS4800         46
#define BPS7200         30
#define BPS9600         22
#define BPS19200        10
#define BPS38400        4
#define BPS57600        2
#define BPS76800        1
#define BPS115200       0


// ----- Write Register Definitions
// Write Register 0 - command register
#define RESET_EXT        0x10
#define SEND_ABORT       0x18
#define RESET_TX_INT     0x28
#define ERROR_RESET      0x30
#define RESET_IUS        0x38
#define RESET_RXCRC      0x40
#define RESET_TXCRC      0x80
#define RESET_TXUNDERRUN 0xc0

// Write Register 1 - Tx/Rx interrupt and data transfer mode definition
#define EXT_INT_ENABLE     0x01
#define TX_INT_ENABLE      0x02
#define PARITY_SPECIAL     0x04
#define RX_INT_1ST_SPECIAL 0x08
#define RX_INT_ALL_SPECIAL 0x10
#define RX_INT_SPECIAL     0x18
#define RX_DMA_REQ         0x60
#define RX_DMA_ENABLE      0x80

// Write Register 2 - interrupt vector

// Write Register 3 - Rx parameters and controls
#define RX_ENABLE        0x01
#define SYNC_INHIBIT     0x02
#define RX_CRC_ENABLE    0x08
#define ENTER_HUNT_MODE  0x10
#define RX5_BIT_CHAR     0x00
#define RX7_BIT_CHAR     0x40
#define RX6_BIT_CHAR     0x80
#define RX8_BIT_CHAR     0xc0

// Write Register 4 - Tx/Rx miscellaneous parameters and modes
#define PARITY_ENABLE    0x01
#define PARITY_EVEN      0x02
#define PARITY_ODD       0x00
#define ONE_STOP_BIT     0x04
#define ONEHALF_STOP_BITS 0x08
#define TWO_STOP_BITS    0x0c
#define MONOSYNC_MODE    0x00
#define BISYNC_MODE      0x10
#define SDLC_MODE        0x20
#define X16_CLOCK        0x40

// Write Register 5 - Tx paramerameters and controls
#define TX_CRC_ENABLE    0x01
#define RTS              0x02
#define CRC_16           0x04
#define TX_ENABLE        0x08
#define BREAK            0x10
#define TX5_BIT_CHAR     0x00
#define TX7_BIT_CHAR     0x20
#define TX6_BIT_CHAR     0x40
#define TX8_BIT_CHAR     0x60
#define DTR              0x80

// Write Register 6 - sync character or SDLC address field

// Write Register 7 - sync character or SDLC flag
#define SDLC_FLAG        0x7e

// Write Register 7 prime - SDLC enhancement register
#define AUTOTX_FLAG      0x01
#define AUTOEOM_RESET    0x02
#define RXCRC            0x20

// Write Register 8 - transmit buffer

// Write Register 9 - master interrupt control
#define VIS              0x01
#define NV               0x02
#define DLC              0x04
#define MIE              0x08
#define STATUS_LO        0x00
#define STATUS_HI        0x10
#define RESET_CH_B       0x40
#define RESET_CH_A       0x80
#define HARDWARE_RESET   0xc0

// Write Register 10 - miscellaneous Tx/Rx control bits
#define FLAG_ON_UNDERRUN  0x00
#define ABORT_ON_UNDERRUN 0x04
#define CRC_PRESET_0      0x00
#define CRC_PRESET_1      0x80

// Write Register 11 - clock mode control
#define TRxC_EQ_BRG      0x02
#define TRxC_IN          0x00
#define TRxC_OUT         0x04
#define RXCLK_RTxC       0x00
#define TXCLK_TRxC       0x08
#define TX_CLK_EQ_BRG    0x10
#define RX_CLK_EQ_BRG    0x40

// Write Register 12 - lower byte of baud generator time constant

// Write Register 13 - upper byte of baud generator time constant

// Write Register 14 - miscellaneous control bits
#define BRG_DISABLE      0x00
#define BRG_ENABLE       0x01
#define BRG_EQ_SYS_CLK   0x02
#define TX_DMAREQ        0x04

// Write Register 15 - external/STATUS interrupt control
#define EN_WR7PRIME      0x01
#define CD_IE            0x08
#define SYNCHUNT_IE      0x10
#define CTS_IE           0x20
#define TXUNDERRUN_IE    0x40
#define BREAK_IE         0x80
#define ABORT_IE         0x80


// ----- Read Register Definitions
// Read Register 0 - Tx/Rx buffer status and external status
#define RX_BUFFER        0x01
#define TX_BUFFER        0x04
#define CD               0x08
#define HUNT_COND        0x10
#define CTS              0x20
#define TXUNDERRUN_COND  0x40
#define BREAK_COND       0x80
#define ABORT_COND       0x80


// Read Register 1 - special receive condition status
#define ALLSENT          0x01
#define PARITY_ERROR     0x10
#define OVERRUN_ERROR    0x20
#define FRAMING_ERROR    0x40
#define CRC_ERROR        0x40
#define END_FRAME        0x80

// Read Register 2 - interrupt vector

// Read Register 3 - interrupt pending status

// Read Register 8 - receive data register
#define SOH              0x01          /* SOH control character */
#define ETX              0x03          /* ETX control character */
#define SYN              0x16          /* SYN control character */

// Read Register 10 - miscellaneous status bits

// Read Register 12 - value stored in WR12

// Read Register 13 - value stored in WR13

// Read Register 15 - value stored in WR15


// ----- Other defines --------------------------------------------------------
#define C2SINT_IO       0xef60   // CTL to SYS interrupt register I/O address
#define C2SINT_HI       0        // value to set interrupt line high
#define C2SINT_LOW      0x0008   // value to set interrupt line low
#define SCCBASE_TYPE    0x80     // base interrupt vector type for line 0 SCC
#define TXRETRY_MAX     3        // max number of Tx frame retries allowed
#define MDMSTAT_IO      0xfe5e   // DSR-RI Register I/O address
#define MAX_CLOSE_DLY   200 * 10 // max time to wait before closing, 10 sec
#define SYNC_CLOSE_DLY  200      // time to wait before close to allow Tx SYNC
                                 // after frame so remote gets frame, 1 sec
#define SCC_CLK         7372800L // SCC clock freq, used to compute data rates


// ----- Function prototypes --------------------------------------------------
void Trace(WORD_T,WORD_T,WORD_T,WORD_T);
void ExecTestModes(void);
void SetTestModeAsync(void);
void SetTestModeHdlc(void);
void SetDtr(LINE_T far *);
void ClrDtr(LINE_T far *);
void DispatchMsg(void);
WORD_T EnqSysMsg(BYTE_T *);
void InitRam(void);
void InitVector(void);
void InitDmaCtrl(void);
void SetVector(WORD_T,void (interrupt far *)());
void InitTimer(WORD_T,UWORD_T,void (interrupt far *)());
void RetryTxHdlc(LINE_T far *);
void StartTxSync(LINE_T far *);
void StartRxBisync(LINE_T far *);
void StartRxHdlc(LINE_T far *);
UDWORD_T FarPtrToAbs(void far *);
void interrupt far Timer1Isr();
void interrupt far DmaTcIsr();
void interrupt far Sys2CtlIsr();
void interrupt far Ln0TbeIsr();
void interrupt far Ln1TbeIsr();
void interrupt far Ln0RcaIsr();
void interrupt far Ln1RcaIsr();
void interrupt far Ln0EscIsr();
void interrupt far Ln1EscIsr();
void interrupt far Ln0SrcIsr();
void interrupt far Ln1SrcIsr();
void cmdTest(BYTE_T far *);
void cmdOpenAsync(BYTE_T far *);
void cmdOpenHdlc(BYTE_T far *);
void cmdOpenBisync(BYTE_T far *);
void cmdClose(BYTE_T far *);
void cmdGetModemInputs(BYTE_T far *);
void cmdSetModemOutputs(BYTE_T far *);
void Close(LINE_T far *);
UDWORD_T GetCtlTime(void);
UWORD_T DecrementSfg(WORD_T);
UWORD_T GetSfg(WORD_T,UWORD_T);
UWORD_T TestSetSfg(WORD_T,UWORD_T);
UWORD_T IncrementSfg(WORD_T);
void TestSfg1(void);
void TestSfg2(void);
void ProcError(UBYTE_T);
WORD_T GetTxqIn(LINE_T far *);
WORD_T GetRxqOut(LINE_T far *);


// ----- Macros ------------------------------------------------------------
#if defined(BORLANDC)
#define ASM asm
#define DIS_INT() {disable(); IntDis++;} // disable interrupts
#define EN_INT() {IntDis--; \
                  if(IntDis == 0) \
                  { \
                    if(!InIsr) enable(); \
                  }}
#define OUTB(PORT,VALUE) outportb(PORT,VALUE) // write a byte to I/O space
#define INB(PORT) inportb(PORT)        // read a byte from I/O space
#define OUTW(PORT,VALUE) outport(PORT,VALUE) // write a word to I/O space
#define INW(PORT) inport(PORT)         // read a word from I/O space
#define MAKE_FP(SEG,OFF) MK_FP(SEG,OFF) // make a far pointer
#endif

#if defined(MSC)
#define ASM _asm
#define DIS_INT() {_disable(); IntDis++;} // disable interrupts
#define EN_INT() {IntDis--; \
                  if(IntDis == 0) \
                  { \
                    if(!InIsr) _enable(); \
                  }}
#define OUTB(PORT,VALUE) _outp(PORT,VALUE) // write a byte to I/O space
#define INB(PORT) _inp(PORT)           // read a byte from I/O space
#define OUTW(PORT,VALUE) _outpw(PORT,VALUE) // write a word to I/O space
#define INW(PORT) _inpw(PORT)          // read a word from I/O space
#define MAKE_FP(SEG,OFF) (((long)SEG << 16) + OFF) // make a far pointer
#endif

#define ISR_PROLOG() InIsr = 1        // put this at beginning of every ISR

#define ISR_EPILOG() {InIsr = 0; \
                   OUTB(INTCTL,EOIVAL);} // put this at end of every ISR

/***************************************************************************
Macro:    SET_RTS
Purpose:  Set RTS modem control output
Call:     SET_RTS(PLN)
          LINE_T far *PLN: Ptr to line structure
Return:   void
Comments: This is used to set the RTS modem control output via SCC WR5.
*/
#define SET_RTS(PLN) {DIS_INT(); \
                     OUTB(PLN->BaseIo,WR5); \
                     OUTB(PLN->BaseIo,pLn->WR5_ | RTS); \
                     EN_INT();}

/***************************************************************************
Macro:    CLR_RTS
Purpose:  Clear RTS modem control output
Call:     CLR_RTS(PLN)
          LINE_T far *PLN: Ptr to line structure
Return:   void
Comments: This is used to clear the RTS modem control output via SCC WR5.
*/
#define CLR_RTS(PLN) {DIS_INT(); \
                     OUTB(PLN->BaseIo,WR5); \
                     OUTB(PLN->BaseIo,pLn->WR5_); \
                     EN_INT();}

/*****************************************************************************
Macro:      GET_CTS_CD
Purpose:    Get the status of a line's CTS and CD modem control inputs
            directly from the SCC.
Call:       GET_CTS_CD(IO)
            WORD_T IO; SCC base I/O address
Return:     UBYTE_T: Status of line's CTS and CD inputs, may be any combination
                     of the CTS and CD flags.  The CTS and CD flags have the
                     same values as the MSG_MDM_CTS and MSG_MDM_CD flags
                     respectively.
*/
#define GET_CTS_CD(IO) (INB(IO) & (CTS | CD))

/*****************************************************************************
Macro:      GET_DSR_RI
Purpose:    Get the status of a line's DSR and RI modem control inputs
            directly from the DSR-RI Register.
Call:       GET_DSR_RI(LN)
            WORD_T LN; Line number
Return:     UBYTE_T: Status of line's DSR and RI inputs, may be any combination
                    of the MDM_MSG_DSR and MDM_MSG_RI flags.
*/
#define GET_DSR_RI(LN) (((~INB(MDMSTAT_IO)) << (2-(LN))) & (MSG_MDM_DSR | MSG_MDM_RI))

/*****************************************************************************
Macro:      GET_DSR
Purpose:    Get the status of a line's DSR modem control input directly from
            the DSR-RI Register.
Call:       GET_DSR(LN)
Return:     BYTE_T: 0 if DSR is OFF, >0 if DSR is ON.
Comments:   The DSR status is obtained from the DSR-RI Register, which
            indiates a signal is ON with a 0, and OFF with a 1, so the register
            contents must be inverted.
*/
#define GET_DSR(LN) ((~INB(MDMSTAT_IO)) & (1 << (LN)))

/*****************************************************************************
Macro:      GET_RI
Purpose:    Get the status of a line's RI modem control input directly from
            the DSR-RI register.
Call:       GET_RI(LN)
Return:     BYTE_T: 0 if RI is OFF, >0 if RI is ON.
Comments:   The RI status is obtained from the DSR-RI Register, which
            indiates a signal is ON with a 0, and OFF with a 1, so the register
            contents must be inverted.
*/
#define GET_RI(LN) ((~INB(MDMSTAT_IO)) & (4 << (LN)))

/*****************************************************************************
Macro:      EN_DMA
Purpose:    Enable a DMA channel
Call:       EN_DMA(MASK)
            BYTE_T MASKVAL; Mask value, all of the lower 4 bits must be 1
               except the bit for the DMA channel being enabled.
Comments:   The DMAU (DMA Unit) must be completely disabled before reading
            and changing the DMK (DMA Mask Register) to prevent a race
            condition between a TC (Terminal Count) setting a bit between the
            DMK read and write operation.
Warnings:   Interrupts must be disabled when calling this macro.
*/
#define EN_DMA(MASKVAL) {OUTB(DDC,DDMA); \
                         OUTB(DMK,INB(DMK) & (MASKVAL)); \
                         OUTB(DDC,0);}

/*****************************************************************************
Macro:      DIS_DMA
Purpose:    Disable a DMA channel
Call:       DIS_DMA(MASK)
            BYTE_T MASKVAL; Mask value, all of the lower 4 bits must be 0
               except the bit for the DMA channel being disabled.
Comments:   The DMAU (DMA Unit) must be completely disabled before reading
            and changing the DMK (DMA Mask Register) to prevent a race
            condition between a TC (Terminal Count) setting a bit between the
            DMK read and write operation.
Warnings:   Interrupts must be disabled when calling this macro.
*/
#define DIS_DMA(MASKVAL) {OUTB(DDC,DDMA); \
                         OUTB(DMK,INB(DMK) | (MASKVAL)); \
                         OUTB(DDC,0);}

/*****************************************************************************
Macro:      EN_INT
Purpose:    Enable interrupts.
Comments:   This function enables interrupts at the processor level by
            setting the interrupt flag in the processor.  This is used to
            reenable interrupts after they have been disabled with DIS_INT.
            EN_INT checks that it was not called from within an ISR before
            enabling interrupts.  This is done because ISRs are not allowed to
            enable interrupts until they exit.  All functions that enable
            interrupts must use EN_INT instead of enabling interrupts directly,
            allowing the user to call any control program function regardless
            of whether the call is in an ISR or not.
*/
// Code is above

/*****************************************************************************
Marco:      GET_DPSEM
Purpose:    Get dual port semaphore.
Comments:   This function is used to guarentee the controller will have
            exclusive access to a portion of dual port memory until RelDpSem()
            is called.  Normally this is only used to protect critical portions
            of memory that can be operated on by both the controller and the
            system.  For this to work properly the system must also also get
            and release the same semaphore using the companion functions
            hiGetDpSem() and hiRelDpSem().
Warnings:   Interrupts must be disabled before calling this function.

            GET_DPSEM and REL_DPSEM must always be used in pairs.

            The time that the semaphore is held must be kept as short
            as possible because the system will spin while it is waiting for
            the semaphore.
*/
#define GET_DPSEM() {*pDpSemCtlAccess = 1; \
                    while(*pDpSemSysAccess){}}

/*****************************************************************************
Macro:      REL_DPSEM
Purpose:    Release dual port semaphore.
Comments:   This function is used to release the dual port semaphore
            after GET_DPSEM is called.
*/
#define REL_DPSEM() (*pDpSemCtlAccess = 0)

/*****************************************************************************
Macro:    INIT_SFG
Purpose:  Initialize all flags in a Status Flag Group.
Call:     kInitSFG(Group,Value)
          WORD_T Group; Status flag group number, identifies which SFG
                        to operate on.
          UWORD_T Value; The value to initialize the SFG to.
Return:   void
Comments: An SFG is a word location in dual port memory which may be read or
          written by either the system or the controller.  Each bit within the
          SFG can be thought of as a seperate flag, so there are 16 flags
          in each SFG.  The number of SFGs that exist is defined by SFG_SIZE.
          SFGs are identified by group number.  Group numbers range from 0 to
          SFG_SIZE - 1.  SFGs are general purpose flag groups.  They can be
          used for intertask or interprocessor communication of any status
          desired.  All functions that operate on SFGs block other tasks
          and the obtain exclusive dual port memory access before accessing the
          SFG until the operation is complete.  This protects the caller
          against races in case multiple tasks or CPUs try to access an SFG
          simultaneusly. The following functions are available for
          manipulating SFGs:  CLR_SFG(), DecrementSfg(), GetSfg(),
          IncrementSfg(), INIT_SFG(), SET_SFG(), and TestSetSfg().  The
          system has analagous functions for operating on SFGs.

Warnings: This macro gets and releases dual port access.
*/
#define INIT_SFG(GROUP,VALUE) {DIS_INT(); \
                              GET_DPSEM(); \
                              pSfg[GROUP] = (VALUE); \
                              REL_DPSEM(); \
                              EN_INT();}

/*****************************************************************************
Function: CLR_SFG
Purpose:  Clear one or more flags in a Status Flag Group.
Call:     CLR_SFG(Group,Mask)
          WORD_T Group; Status flag group number, identifies which SFG
                        to operate on.
          UWORD_T Mask; Clear mask.  All flags set to 1 in Mask will be
                        cleared, other flags will be unchanged.
Return:   void
Comments: See INIT_SFG.
Warnings: This macro gets and releases dual port access.
*/
#define CLR_SFG(GROUP,MASK) {DIS_INT(); \
                            GET_DPSEM(); \
                            pSfg[GROUP] &= ~(MASK); \
                            REL_DPSEM(); \
                            EN_INT();}

/*****************************************************************************
Macro:      SET_SFG
Purpose:    Set one or more flags in a Status Flag Group.
Call:       SET_SFG(Group,Mask)
            WORD_T Group;  Status flag group number, identifies which SFG
                           to operate on.
            UWORD_T Mask;  Or mask.  All flags set to 1 in Mask will be
                           set in the flag group, other flags will be
                           unchanged.
Return:     void
Comments:   See INIT_SFG.
Warnings:   This mscro gets and releases dual port access.
*/
#define SET_SFG(GROUP,MASK) {DIS_INT(); \
                            GET_DPSEM(); \
                            pSfg[GROUP] |= (MASK); \
                            REL_DPSEM(); \
                            EN_INT();}









