/******************************************************************************
File:     test.c
Purpose:  Test program running on system to check out operation of Hostess i/S2
          tool kit control program.
Company:  Comtrol Corporation
Author:   Craig Harrison
Comments: This program takes 4 command line parameters:
             Parameter 1: Test number to run
             Parameter 2: First line number to test
             Parameter 3: Last line number to test
             Parameter 4: Test specific parameter - see individual test
                          function comments about nTestParm.
Ident:    $Id$
*******************************************************************************
 Copyright 1994 Comtrol Corporation. All rights reserved. Subject to
 developers license agreement.
******************************************************************************/
#include <stdio.h>
#include <dos.h>
#include <process.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>

#include "hilib.h"
#include "test.h"

long DataRates[NUM_DATARATES] =        /* table of synchronous data rates */
{
   0L,                                 /* 0 - unused */
   9600L,                              /* 1 - 9.6k bps */
   19200L,                             /* 2 - 19.2k bps */
   38400L,                             /* 3 - 38.4k bps*/
   57600L,                             /* 4 - 57.6k bps */
   64000L,                             /* 5 - 64k bps */
   76800L,                             /* 6 - 76.8k bps */
   100000L,                            /* 7 - 100k bps */
   115200L,                            /* 8 - 115.2k bps */
   128000L,                            /* 9 - 128k bps */
   192000L,                            /* 10 - 192k bps */
   256000L,                            /* 11 - 256k bps */
   384000L                             /* 12 - 384k bps */
};

int SysMsgAvail;                       /* system messages available */
int nTest;
int nFirstLn, nLastLn;
LNINFO_T LnInfo[NUM_LN];

MODEM_T MdmState[NUM_MDMSTATE] =
{  {0,OFF,OFF,0},
   {MSG_MDM_DTR,ON,OFF,MSG_MDM_DSR | MSG_MDM_CD},
   {MSG_MDM_RTS,OFF,ON,MSG_MDM_CTS | MSG_MDM_RI},
   {MSG_MDM_DTR | MSG_MDM_RTS,ON,ON,MSG_MDM_DSR | MSG_MDM_CD | MSG_MDM_CTS | MSG_MDM_RI}
};

int nTestParm;                         /* 4th command line parameter */
BYTE_T sOutBuf[OUTBUF_SIZE+1];
BYTE_T s1[] = "This is a test of the Comtrol Hostess i/S2 Developer Toolkit, it's only a test!!";
BYTE_T s2[] = "[000 ][000 ][0000 ]This is a test of the Comtrol Hostess i/S2 DevelopersToolkit!";

char *snLn, *sSeq, *sLen;              /* Ptrs to s2[] line, seq num, length */

main(int argc,char *argv[])
{
   int i,j;

   /* Get command line parameters */
   if(argc != 5)
   {
      printf("Usage: test <Test number> <First line> <Last line> <Debug parm>\n");
      exit(1);
   }
   nTest = atoi(argv[1]);
   nFirstLn = atoi(argv[2]);
   nLastLn = atoi(argv[3]);
   nTestParm = atoi(argv[4]);
   printf("Test Number = %d, First Line = %d, Last Line = %d\n",nTest,
           nFirstLn,nLastLn);
   if(nLastLn >= NUM_LN)
   {
      printf("Last nLnice parameter out of range, check NUM_LN in TEST.H\n");
      exit(1);
   }

   /* Configure dual port memory */
   hiInitDpm();

   /* Initialize line information structures */
   memset(LnInfo,0,sizeof(LNINFO_T) * NUM_LN);

   /* Initialize random number generator */
   srand(1);

   /* Initialize output buffer */
   for(i = 0;i < OUTBUF_SIZE;i += 80)
   {
      j = 80;
      if(i + 80 > OUTBUF_SIZE)         /* this copy reaches end of buffer */
         j = OUTBUF_SIZE - i;
      strncpy(&sOutBuf[i],s1,j);
   }
   snLn = &sOutBuf[1];                 /* pointers into s2[] */
   sSeq = &sOutBuf[7];
   sLen = &sOutBuf[13];

   /* Initialize interrupt vector and PIC */
   hiInstallIrq(IRQ_NUM,IntHandler);

   /* Select test to run */
   switch(nTest)
   {
      case 0:
         TestCtrlMsg();
         break;
      case 1:
         TestOpenAsync_Close();
         break;
      case 2:
         TestModemState();
         break;
      case 3:
         TestWrRdAsync();
         break;
      case 4:
         TestOpenHdlc_Close();
         break;
      case 5:
         TestWrHdlc();
         break;
      case 6:
         TestRdHdlc();
         break;
      case 7:
         TestWrRdHdlc();
         break;
      case 8:
         TestOpenBisync_Close();
         break;
      case 9:
         TestWrBisync();
         break;
      case 10:
         TestRdBisync();
         break;
      case 11:
         TestWrRdBisync();
         break;
      case 12:
         TestSfg1();
         break;
      case 13:
         TestSfg2();
         break;
      case 14:
         TestAckAsync();
         break;
      default:
         printf("Invalid test number\n");
         hiRestoreIrq();
         exit(1);
   }
   return(0);
}

/******************************************************************************
Function: TestCtrlMsg
Purpose:  Test send controller message with and without reply.
Comments: While running display [OK].
*/
void TestCtrlMsg(void)
{
   static char sTestMsg[MSG_LEN]=
   {
      CTLCMD_TEST,   /* command number parameter */
      0,             /* unused */
      0              /* 0 = no reply, 1 = reply */
   };
   static BYTE_T sReplyBuf[REPLY_LEN];

   while(!kbhit())
   {
      /* Verify send message without reply */
      sTestMsg[2] = 0;                 /* no reply requested */
      if(hiEnqCtlMsg(sTestMsg,NULL) != 1) /* send msg, fail */
      {
         printf("Send message failure\n");
         ExitTest();
      }

      /* Verify timeout failure if reply requrested but not received */
      sTestMsg[2] = 0;                 /* no reply requested */
      if(hiEnqCtlMsg(sTestMsg,sReplyBuf) != -1) /* send msg, no timeout */
      {
         printf("No Timeout failure\n");
         ExitTest();
      }

      /* Verify reply received */
      sTestMsg[2] = 1;                 /* reply requested */
      if(hiEnqCtlMsg(sTestMsg,sReplyBuf) != 1) /* send msg, fail */
      {
         printf("Timeout failure\n");
         ExitTest();
      }
      else                             /* success, verify reply buf */
      {
         if(strcmp(sReplyBuf,"OK") != 0)
         {
            printf("Bad reply\n");
            ExitTest();
         }
      }
      printf("[OK]");
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestOpenAsync_Close
Purpose:  Test hiOpenAsync() and hiClose(), sequential opens and closes,
          verify modem control after open.
Comments: Compile control program with #define LNX_ASYNC set to 1.

          Loopback plugs must be installed connecting RTS-CTS-RI and
          DTR-DSR-CD.

          After each set of opens the modem control outputs vary sequentially
          through NUM_MDMSTATE possible states.

          While running display [O-X] on open and [C-X] on close, where
          "X" is replaced by the line number.
*/
void TestOpenAsync_Close(void)
{
   int nLn;                            /* line number */
   static int nMdm = 0;                /* modem control state number */
   int nMdmIn;                         /* modem control inputs */

   while(!kbhit())
   {
      /* Open all lines */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         /* Open line */
         if(!hiOpenAsync(nLn,MSG_BAUD_9600,MSG_PAR_NONE,MSG_DATABIT_8,
                         MSG_STOPBIT_1,MdmState[nMdm].nOut))
         {
            printf("Open Failure - Line %d\n",nLn);
            ExitTest();
         }
         LnInfo[nLn].nLnOpen = TRUE;
         if(LnInfo[nLn].nLoop1 >= PRINTCNT)
            printf("[O-%d]",nLn);
         LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);

         /* Check modem inputs */
         nMdmIn = hiGetModem(nLn);
         if(nMdmIn != MdmState[nMdm].nIn)
         {
            printf("Modem Failure - Line %d, State num %d, Inputs %x\n",
                   nLn,nMdm,nMdmIn);
            ExitTest();
         }
      }
      nMdm = (nMdm + 1) % NUM_MDMSTATE; /* next modem state number */

      /* Close all lines */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(!hiClose(nLn))
         {
            ExitTest();
         }
         LnInfo[nLn].nLnOpen = FALSE;
         if(LnInfo[nLn].nLoop2 >= PRINTCNT)
            printf("[C-%d]",nLn);
         LnInfo[nLn].nLoop2 = (LnInfo[nLn].nLoop2 + 1) % (PRINTCNT + 1);
      }
      hiDelay(100);                 /* allow close to finish */
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestModemState
Purpose:  Test hiGetModem() and hiGetModem() after async, HDLC, or Bisync open.
Comments: Compile control program with proper communications mode #defines
          set to 1.

          Set 4th command line parameter (nTestParm) to the type of open
          desired, as follows:
             0: Async
             1: HDLC - not supported in control program
             2: Bisync - not supported in control program

          Loopback plugs must be installed connecting RTS-CTS-RI and
          DTR-DSR-CD.

          Modem control outputs vary randomly through NUM_MDMSTATE possible
          states.

          While running display [M-X-Y], "X" is replaced by the line number
          and "Y" by modem state number.

          This test was originally checked out with the HDLC and Bisync control
          programs, but then the modem control support was removed from the
          CPC.C LnXEscIsr() functions to improve performance.  This test will
          not currently run unless #define LNX_ASYNC is set to 1.
*/
void TestModemState(void)
{
   int t;                           /* test state */
   int nLn;                         /* line number */
   int nIn;                         /* modem control inputs */

   /* Open all lines with RTS and DTR OFF, verify modem inputs */
   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      switch(nTestParm)             /* do proper type of open */
      {
         case 0:                    /* Async open */

            if(!hiOpenAsync(nLn,MSG_BAUD_9600,MSG_PAR_NONE,MSG_DATABIT_8,
                            MSG_STOPBIT_1,MdmState[0].nOut))
            {
               printf("Open Failure - Line %d\n",nLn);
               ExitTest();
            }
            break;
         case 1:                    /* HDLC open */
            if(!hiOpenHdlc(nLn,MdmState[0].nOut,MSG_CLK_1,0))
            {
               printf("Open Failure - Line %d\n",nLn);
               ExitTest();
            }
            break;
         case 2:                    /* Bisync open */
            if(!hiOpenBisync(nLn,MdmState[0].nOut,MSG_CLK_1,0))
            {
               printf("Open Failure - Line %d\n",nLn);
               ExitTest();
            }
            break;
         default:
            printf("Bad command line parm 4\n");
            ExitTest();
      }
      LnInfo[nLn].nLnOpen = 1;
      nIn = hiGetModem(nLn);
      if(nIn != MdmState[0].nIn)
      {
         printf("Modem Failure on Open - Line %d, Inputs %x\n",nLn,nIn);
         ExitTest();
      }
   }

   while(1)
   {
      if(kbhit())
      {
         getch();
         ExitTest();
      }

      /* Randomly change modem outputs on random line, verify inputs */
      nLn = GetRandom(nFirstLn,nLastLn); /* choose random line */
      t = GetRandom(0,NUM_MDMSTATE-1); /* choose random test state */
      LnInfo[nLn].nCnt1 = t;
      if(!hiChangeModem(nLn,MdmState[t].nDtr,MdmState[t].nRts))
      {
         printf("aaChangeModem() Failure - Line %d\n",nLn);
         ExitTest();
      }
      hiDelay(100L);                   /* allow for RS-232 delay */
      nIn = hiGetModem(nLn);
      if(nIn != MdmState[t].nIn)
      {
         printf("Modem Failure - Line %d, State = %d, Inputs %x\n",nLn,t,nIn);
         ExitTest();
      }
      if(LnInfo[nLn].nLoop1 >= PRINTCNT)
         printf("[S-%d-%d]",nLn,t);

      /* Verify the modem lines for a random device */
      nLn = GetRandom(nFirstLn,nLastLn);  /* choose random device */
      t = LnInfo[nLn].nCnt1;              /* get current test state */
      nIn = hiGetModem(nLn);
      if(nIn != MdmState[t].nIn)
      {
         printf("Random Modem Failure - Line %d, State = %d, Inputs %x\n",
                nLn,t,nIn);
         ExitTest();
      }

      if(LnInfo[nLn].nLoop1 >= PRINTCNT)
         printf("[V-%d-%d]",nLn,t);
      LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
   }
}

/******************************************************************************
Function: TestWrRdAsync
Purpose:  Test hiWriteAsync(), hiReadAsync() with random length write/reads on
          random lines.  The read will verify the line, sequence number, and
          length.
Comments: Compile control program with #define LNX_ASYNC set to 1.

          Loopback plugs must be installed connecting TX-RX.

          Set #define TWRA_WRSIZE to the largest write to be done.
          Set #define TWRA_BAUD to the desired MSG_BAUD_X flag.

          While running displays:
            [NW-X-Y-Z] on first write of buffer
            [W-X-Y] on subsequent writes of buffer
            [NR-X-Y-Z] on first read transmitted buffer
            [R-X-Y] on subsequent reads of transmitted buffer
          where
            X = Line number
            Y = Data count
            Z = Sequence number
*/
void TestWrRdAsync(void)
{
   int i, nLn;
   int nCurTxCnt;           /* bytes written by current write */
   int nCurRxCnt;           /* Rx count from current read */
   int nRxCnt;              /* accumulated Rx count */
   char *pBuf;
   int nRecLn;              /* received line number */
   int nRecSeq;             /* received sequence number */
   int nRecLen;             /* received string length */
   static char sTemp[22];

   /* Initialize output buffers, open lines */
   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      pBuf = LnInfo[nLn].sTxBuf;
      pBuf[0] = '\r';                   /* put CR/LF at front of Tx string */
      pBuf[1] = '\n';
      strncpy(pBuf+2,s2,80);  /* output str has line num, seq num, & length */
      pBuf[82] = 0;                     /* put NULL in output string */
      strncpy(pBuf+83,sOutBuf,BUF_SIZE - 84);
      if(!hiOpenAsync(nLn,TWRA_BAUD,MSG_PAR_NONE,MSG_DATABIT_8,
                      MSG_STOPBIT_1,MSG_MDM_DTR | MSG_MDM_RTS))
      {
         printf("Open Failure - Line %d\n",nLn);
         ExitTest();
      }
      LnInfo[nLn].nLnOpen = TRUE;
   }
   sTemp[21] = '\0';

   while(1)
   {
      if(kbhit())
      {
         getch();
         ExitTest();
      }
      ProcessMsg();                    /* check for system messages */

      /* Write data */
      nLn = GetRandom(nFirstLn,nLastLn); /* get random line number */
      if(hiGetTxByteCnt(nLn) < TXQ_SIZE/5) /* avoid zillions of small writes */
      {
         pBuf = LnInfo[nLn].sTxBuf;          /* get ptr to Tx buffer */
         if(LnInfo[nLn].nTxCnt >= LnInfo[nLn].nTxTotal) /* start new write */
         {
            LnInfo[nLn].nTxTotal = GetRandom(21,TWRA_WRSIZE); /* write size */
            sprintf(&pBuf[3],"%-3d",nLn); /* fill in string */
            sprintf(&pBuf[9],"%-3d",LnInfo[nLn].nTxSeq);
            sprintf(&pBuf[15],"%-4d",LnInfo[nLn].nTxTotal);
            for(i = 4;i <= 19;i++)      /* replace NULLs with printable char */
            {
               if(pBuf[i] == '\0')
                  pBuf[i] = ' ';
            }
            LnInfo[nLn].nTxCnt = hiWriteAsync(nLn,pBuf,LnInfo[nLn].nTxTotal);
            if(LnInfo[nLn].nTxCnt > 0)
            {
               if(LnInfo[nLn].nLoop1 >= PRINTCNT)
                  printf("[NW-%d-%d-%d]",nLn,LnInfo[nLn].nTxCnt,LnInfo[nLn].nTxSeq);
               LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
            }
            LnInfo[nLn].nTxSeq = (LnInfo[nLn].nTxSeq + 1) % 1000;
         }
         else                             /* finish a previous write */
         {
            nCurTxCnt = hiWriteAsync(nLn,&pBuf[LnInfo[nLn].nTxCnt],
                                     LnInfo[nLn].nTxTotal - LnInfo[nLn].nTxCnt);
            if(nCurTxCnt > 0)
            {
               LnInfo[nLn].nTxCnt += nCurTxCnt;
               if(LnInfo[nLn].nLoop1 >= PRINTCNT)
                  printf("[W-%d-%d]",nLn,nCurTxCnt);
               LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
            }
         }
      }
      hiDelay(50);   /* avoid excessive DP mem hits, see hiMemCpy() comments */

      /* Read data.  The 1st 21 bytes contain the line num, seq num, and
         length of the receive string.  This gets 1st 21 bytes first, then
         the balance of the string gets read. */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         pBuf = LnInfo[nLn].sRxBuf;         /* get ptr to Rx buffer */
         if(LnInfo[nLn].nRxTotal == 0)      /* start a new read */
         {             /* this reads 1st 21 bytes to get line, seq, & length */

            if(hiGetRxByteCnt(nLn) >= 21) /* 21 bytes avail */
            {
               nCurRxCnt = hiReadAsync(nLn,pBuf,21);
               if(nCurRxCnt != 21)
               {
                  printf("New Read Failure - Line %d, Cnt %d\n",nLn,nCurRxCnt);
                  ExitTest();
               }
               if(LnInfo[nLn].nLoop2 >= PRINTCNT)
               {
                  printf("[NR-%d-%d-%d]",nLn,nCurRxCnt,LnInfo[nLn].nRxSeq);
               }
               LnInfo[nLn].nRxCnt = nCurRxCnt;
               strncpy(sTemp,pBuf,21); /* save 1st 21 bytes for error msg */
               for(i = 4;i <= 19;i++)  /* replace spaces with NULLs */
               {
                  if(pBuf[i] == ' ')
                     pBuf[i] = '\0';
               }
               nRecLn = atoi(&pBuf[3]); /* get message info */
               nRecSeq = atoi(&pBuf[9]);
               nRecLen = atoi(&pBuf[15]);
               if((nRecLn != nLn) ||    /* bad Rx line number */
                  (nRecSeq != LnInfo[nLn].nRxSeq) || /* bad Rx sequence num */
                  (nRecLen < 21) ||                  /* bad Rx string length */
                  (nRecLen > TWRA_WRSIZE))
               {
                  printf("\nRx Fail [nLn:%d:%d][Seq:%d:%d][Len:21+:%d]\n",
                         nLn,nRecLn,LnInfo[nLn].nRxSeq,nRecSeq,nRecLen);
                  printf("Err Str:%s\n",sTemp);    /* dump saved string */
                  printf("Err Str Hex:");
                  for(i = 0;i < 21;i++)
                     printf("%x ",sTemp[i]);
                  printf("\n");
                  for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
                  {
                     while(hiReadAsync(nLn,LnInfo[nLn].sRxBuf,BUF_SIZE));
                  }
                  ExitTest();
               }
               LnInfo[nLn].nRxTotal = nRecLen;
               LnInfo[nLn].nRxSeq = (LnInfo[nLn].nRxSeq + 1) % 1000;
            }
            else                           /* don't have 21 bytes avail yet */
               hiDelay(10);                /* avoid excessive DP memory hits */
         }
         if(LnInfo[nLn].nRxTotal > 0)      /* read balance of string */
         {
            nCurRxCnt = hiGetRxByteCnt(nLn); /* number Rx bytes available */
            nRxCnt = LnInfo[nLn].nRxCnt;

            /* To avoid zillions of reads returning just a few bytes don't
               read unless we have at least 50 available, or this is end of
               a transmit string */
            if(nCurRxCnt > 50 || nCurRxCnt >= LnInfo[nLn].nRxTotal - nRxCnt)
            {
               nCurRxCnt = hiReadAsync(nLn,pBuf+nRxCnt,
                                       LnInfo[nLn].nRxTotal - nRxCnt);
               if(LnInfo[nLn].nLoop2 >= PRINTCNT)
               {
                  printf("[R-%d-%d]",nLn,nCurRxCnt);
               }
               LnInfo[nLn].nLoop2 = (LnInfo[nLn].nLoop2 + 1) % (PRINTCNT + 1);
               nRxCnt += nCurRxCnt;
               LnInfo[nLn].nRxCnt = nRxCnt;
               if(nRxCnt >= LnInfo[nLn].nRxTotal)   /* entire string read */
               {
                  LnInfo[nLn].nRxCnt = 0;
                  LnInfo[nLn].nRxTotal = 0;
               }
            }
         }
         hiDelay(50); /* avoid excessive DP mem hits, see hiMemCpy() comments */

      }
   }
} /* end TestWrRdAsync() */

/******************************************************************************
Function: TestOpenHdlc_Close
Purpose:  Test hiOpenHdlc() and hiClose(), sequential opens and closes,
          verify modem control after open.
Comments: Compile control program with #define LNX_HDLC set to 1.

          Loopback plugs must be installed connecting RTS-CTS-RI and
          DTR-DSR-CD.

          After each set of opens the modem control outputs vary sequentially
          through NUM_MDMSTATE possible states.

          While running display [O-X] on open and [C-X] on close, where
          "X" is replaced by the line number.
*/
void TestOpenHdlc_Close(void)
{
   int nLn;                            /* line number */
   static int nMdm = 0;                /* modem control state number */
   int nMdmIn;                         /* modem control inputs */

   while(!kbhit())
   {
      /* Open all lines */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         /* Open line */
         if(!hiOpenHdlc(nLn,MdmState[nMdm].nOut,MSG_CLK_2,9600))
         {
            printf("Open Failure - Line %d\n",nLn);
            ExitTest();
         }
         LnInfo[nLn].nLnOpen = TRUE;
         if(LnInfo[nLn].nLoop1 >= PRINTCNT)
            printf("[O-%d]",nLn);
         LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);

         /* Check modem inputs */
         nMdmIn = hiGetModem(nLn);
         if(nMdmIn != MdmState[nMdm].nIn)
         {
            printf("Modem Failure - Line %d, State num %d, Inputs %x\n",
                   nLn,nMdm,nMdmIn);
            ExitTest();
         }
      }
      nMdm = (nMdm + 1) % NUM_MDMSTATE; /* next modem state number */

      /* Close all lines */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(!hiClose(nLn))
         {
            ExitTest();
         }
         LnInfo[nLn].nLnOpen = FALSE;
         if(LnInfo[nLn].nLoop2 >= PRINTCNT)
            printf("[C-%d]",nLn);
         LnInfo[nLn].nLoop2 = (LnInfo[nLn].nLoop2 + 1) % (PRINTCNT + 1);
      }
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestWrHdlc
Purpose:  Test hiWriteSync() on HDLC open with fixed length writes on
          sequential lines.
Comments: Compile control program with #define LNX_HDLC set to 1.

          If the remote DCE is supplying Tx clock, set #define TWH_CLK to
          MSG_CLK_1.

          If no remote DCE is supplying Tx clock it must be generated locally,
          so set #define TWH_CLK to MSG_CLK_2.

          Set #define TWH_WRSIZE to the number of bytes to send on each write.

          Set #define TWH_DATARATE to the data rate in BPS.  If the
          4th command line parameter (nTestParm) is set to 0 TWH_DATARATE will
          be used.  Otherwise nTestParm is an index into DataRates[] and
          the data rate is selected from there.

          While running display [W-X] on successful write, where "X" is
          replaced by the line number.
*/
void TestWrHdlc(void)
{
   int nLn;                            /* line number */
   long DataRate;

   /* Get the data rate from the table or from #define TWH_DATARATE */
   if(nTestParm < 0 || nTestParm >= NUM_DATARATES)
      nTestParm = 0;
   if(nTestParm == 0)
      DataRate = TWH_DATARATE;
   else
      DataRate = DataRates[nTestParm];

   /* Open all lines */
   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      if(!hiOpenHdlc(nLn,MSG_MDM_DTR | MSG_MDM_RTS,TWH_CLK,DataRate))
      {
         printf("Open Failure - Line %d\n",nLn);
         ExitTest();
      }
      LnInfo[nLn].nLnOpen = TRUE;
   }

   while(!kbhit())
   {
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(LnInfo[nLn].nTxBufCnt < TXBQ_SIZE) /* empty Tx buffers avail */
         {
            if(hiWriteSync(nLn,sOutBuf,TWH_WRSIZE)) /* successful write */
            {
               LnInfo[nLn].nTxBufCnt = hiGetTxBufCnt(nLn);
               if(LnInfo[nLn].nLoop1 >= PRINTCNT)
                  printf("[W-%d]",nLn);
               LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
            }
            else                       /* write failed */
            {
               printf("Write Failure - Line %d\n",nLn);
               ExitTest();
            }
         }
      }
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestRdHdlc
Purpose:  Test hiReadSync() on HDLC open on sequential lines.
Comments: Compile control program with #define LNX_HDLC set to 1.

          This test requires that the Rx clock be DCE sourced.

          While running display [R-X-Y-Z] each time a frame is received, where
          X is the line number, Y is the number of bytes in the received
          frame, and Z is the first nTestParm bytes (in hex) in the
          received frame.

          Set 4th command line parameter (nTestParm) to the number of
          received bytes to display.
*/
void TestRdHdlc(void)
{
   int nLn;                            /* line number */
   int nRx;                            /* number of received data bytes */
   BYTE_T *pBuf;                       /* ptr to buffer for Rx frame */
   int nDisplay;                       /* number of Rx bytes to display */
   int i;

   /* Open all lines */
   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      if(!hiOpenHdlc(nLn,MSG_MDM_DTR | MSG_MDM_RTS,MSG_CLK_1,0))
      {
         printf("Open Failure - Line %d\n",nLn);
         ExitTest();
      }
      LnInfo[nLn].nLnOpen = TRUE;
   }

   while(!kbhit())
   {
      ProcessMsg();                    /* check for system messages */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(LnInfo[nLn].nRxBufCnt)     /* an Rx buffer is available */
         {
            pBuf = LnInfo[nLn].sRxBuf;
            nRx = hiReadSync(nLn,pBuf);
            LnInfo[nLn].nRxBufCnt = hiGetRxBufCnt(nLn); /* update Rx buf cnt */
            if(nRx)
            {
               if(LnInfo[nLn].nLoop1 >= PRINTCNT)
               {
                  if(nTestParm == 0)
                     printf("[R-%d-%d]",nLn,nRx);
                  else
                  {
                     if(nRx < nTestParm)
                        nDisplay = nRx;
                     else
                        nDisplay = nTestParm;
                     printf("[R-%d-%d-",nLn,nRx);
                     for(i = 0;i < nDisplay;i++)
                        printf(" %x",pBuf[i]);
                     printf("]\n");
                  }
               }
               LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
            }
         }
      }
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestWrRdHdlc
Purpose:  Test hiWriteSync(), hiReadSync() on HDLC open with random length
          writes/reads on random writes lines, sequential read lines.  The
          read will verify the line, sequence number, and length.
Comments: Compile control program with #define LNX_HDLC set to 1.

          Loopback plugs must be installed connecting TX-RX and TxClk-RxClk.

          Set #define TWRS_WRSIZE to the max number of bytes to send on each
          write.

          Set #define TWRS_DATARATE to the data rate in BPS.  If the
          4th command line parameter (nTestParm) is set to 0 TWRS_DATARATE will
          be used.  Otherwise nTestParm is an index into DataRates[] and
          the data rate is selected from there.

          While running display [W-X-Y-Z] on successful write and [R-X-Y-Z] on
          successful read, where "X" is replaced by the line number, Y by the
          sequence number, and Z by the data count.
*/
void TestWrRdHdlc(void)
{
   int i,nLn;
   char *pBuf;
   int nTxCnt,nRxCnt;       /* Tx and Rx byte counts */
   int nRecLn;              /* received line number */
   int nRecSeq;             /* received sequence number */
   int nRecLen;             /* received string length */
   static char sTemp[22];
   long DataRate;

   /* Get the data rate from the table or from #define TWH_DATARATE */
   if(nTestParm < 0 || nTestParm >= NUM_DATARATES)
      nTestParm = 0;
   if(nTestParm == 0)
      DataRate = TWH_DATARATE;
   else
      DataRate = DataRates[nTestParm];

   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      pBuf = LnInfo[nLn].sTxBuf;
      pBuf[0] = '\r';                   /* put CR/LF at front of Tx string */
      pBuf[1] = '\n';
      strncpy(pBuf+2,s2,80);  /* output str has line num, seq num, & length*/
      pBuf[82] = 0;                     /* put NULL in output string */
      strncpy(pBuf+83,sOutBuf,BUF_SIZE - 84);
      if(!hiOpenHdlc(nLn,MSG_MDM_DTR | MSG_MDM_RTS,MSG_CLK_2,DataRate))
      {
         printf("Open Failure - Line %d\n",nLn);
         ExitTest();
      }
      LnInfo[nLn].nLnOpen = TRUE;
   }

   sTemp[21] = '\0';
   while(1)
   {
      if(kbhit())
      {
         getch();
         ExitTest();
      }

      /* Write data */
      ProcessMsg();                    /* check for system messages */
      nLn = GetRandom(nFirstLn,nLastLn); /* get random line number */
      if(LnInfo[nLn].nTxBufCnt < TXBQ_SIZE) /* controller Tx buffers avail */
      {
         pBuf = LnInfo[nLn].sTxBuf;    /* get ptr to system Tx buffer */
         nTxCnt = GetRandom(21,TWRS_WRSIZE); /* write size */
         sprintf(&pBuf[3],"%-3d",nLn); /* fill in string */
         sprintf(&pBuf[9],"%-3d",LnInfo[nLn].nTxSeq);
         sprintf(&pBuf[15],"%-4d",nTxCnt);
         for(i = 4;i <= 19;i++)      /* replace NULLs with printable char */
         {
            if(pBuf[i] == '\0')
               pBuf[i] = ' ';
         }
         if(hiWriteSync(nLn,pBuf,nTxCnt) != 1)
         {
            printf("Write Failure - Line %d\n",nLn);
            ExitTest();
         }
         LnInfo[nLn].nTxBufCnt = hiGetTxBufCnt(nLn);
         if(LnInfo[nLn].nLoop1 >= PRINTCNT)
            printf("[W-%d-%d-%d]",nLn,LnInfo[nLn].nTxSeq,nTxCnt);
         LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
         LnInfo[nLn].nTxSeq = (LnInfo[nLn].nTxSeq + 1) % 1000;
      }

      /* Read data.  The 1st 21 bytes contain the line num, seq num, and
         length of the receive string. */
      ProcessMsg();                    /* check for system messages */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(LnInfo[nLn].nRxBufCnt)        /* Rx buffers are available */
         {
            pBuf = LnInfo[nLn].sRxBuf;    /* get ptr to Rx buffer */
            nRxCnt = hiReadSync(nLn,pBuf); /* read a frame of data */
            LnInfo[nLn].nRxBufCnt = hiGetRxBufCnt(nLn);
            if(nRxCnt > 0)                /* read successful */
            {
               if(nRxCnt < 21)            /* frame too short */
               {
                  printf("Rx frame too short - Line %d, Cnt %d\n",nLn,nRxCnt);
                  ExitTest();
               }
               strncpy(sTemp,pBuf,21); /* save 1st 21 bytes for error msg */
               for(i = 4;i <= 19;i++)     /* replace spaces with NULLs */
               {
                  if(pBuf[i] == ' ')
                  pBuf[i] = '\0';
               }
               nRecLn = atoi(&pBuf[3]);   /* get message info */
               nRecSeq = atoi(&pBuf[9]);
               nRecLen = atoi(&pBuf[15]);
               if((nRecLn != nLn) ||      /* bad Rx line number */
                  (nRecSeq != LnInfo[nLn].nRxSeq) || /* bad Rx sequence number */
                  (nRecLen != nRxCnt) ||  /* bad Rx string length */
                  (nRecLen > TWRA_WRSIZE))
               {
                  printf("\nRx Fail [nLn:%d:%d][Seq:%d:%d][Len:%d:%d]\n",
                         nLn,nRecLn,LnInfo[nLn].nRxSeq,nRecSeq,nRxCnt,nRecLen);
                  printf("Err Str:%s\n",sTemp); /* dump saved string */
                  printf("Err Str Hex:");
                  for(i = 0;i < 21;i++)
                     printf("%x ",sTemp[i]);
                  printf("\nCR to close and exit:  ");
                  while(!kbhit());
                  getch();
                  printf("\n");
                  ExitTest();
               }
               if(LnInfo[nLn].nLoop2 >= PRINTCNT)
                  printf("[R-%d-%d-%d]",nLn,LnInfo[nLn].nRxSeq,nRxCnt);
               LnInfo[nLn].nLoop2 = (LnInfo[nLn].nLoop2 + 1) % (PRINTCNT + 1);
               LnInfo[nLn].nRxSeq = (LnInfo[nLn].nRxSeq + 1) % 1000;
            }
            else
            {
               printf("  Zero length read - Line %d,RxSeq %d\n",nLn,LnInfo[nLn].nRxSeq);
            }
         }
      }
   }
}

/******************************************************************************
Function: TestOpenBisync_Close
Purpose:  Test hiOpenBisync() and hiClose(), sequential opens and closes,
          verify modem control after open.
Comments: Compile control program with #define LNX_BISYNC set to 1.

          Loopback plugs must be installed connecting RTS-CTS-RI and
          DTR-DSR-CD.

          After each set of opens the modem control outputs vary sequentially
          through NUM_MDMSTATE possible states.

          While running display [O-X] on open and [C-X] on close, where
          "X" is replaced by the line number.
*/
void TestOpenBisync_Close(void)
{
   int nLn;                            /* line number */
   static int nMdm = 0;                /* modem control state number */
   int nMdmIn;                         /* modem control inputs */

   while(!kbhit())
   {
      /* Open all lines */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         /* Open line */
         if(!hiOpenBisync(nLn,MdmState[nMdm].nOut,MSG_CLK_2,9600))
         {
            printf("Open Failure - Line %d\n",nLn);
            ExitTest();
         }
         LnInfo[nLn].nLnOpen = TRUE;
         if(LnInfo[nLn].nLoop1 >= PRINTCNT)
            printf("[O-%d]",nLn);
         LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);

         /* Check modem inputs */
         nMdmIn = hiGetModem(nLn);
         if(nMdmIn != MdmState[nMdm].nIn)
         {
            printf("Modem Failure - Line %d, State num %d, Inputs %x\n",
                   nLn,nMdm,nMdmIn);
            ExitTest();
         }
      }
      nMdm = (nMdm + 1) % NUM_MDMSTATE; /* next modem state number */

      /* Close all lines */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(!hiClose(nLn))
         {
            ExitTest();
         }
         LnInfo[nLn].nLnOpen = FALSE;
         if(LnInfo[nLn].nLoop2 >= PRINTCNT)
            printf("[C-%d]",nLn);
         LnInfo[nLn].nLoop2 = (LnInfo[nLn].nLoop2 + 1) % (PRINTCNT + 1);
      }
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestWrBisync
Purpose:  Test hiWriteSync() on BISYNC open with fixed length writes on
          sequential lines.
Comments: Compile control program with #define LNX_BISYNC set to 1.

          If the remote DCE is supplying Tx clock, set #define TWB_CLK to
          MSG_CLK_1.

          If no remote DCE is supplying Tx clock it must be generated locally,
          so set #define TWB_CLK to MSG_CLK_2.

          Set #define TWB_WRSIZE to the number of bytes to send on each write.

          Set #define TWB_DATARATE to the data rate in BPS.  If the
          4th command line parameter (nTestParm) is set to 0 TWB_DATARATE will
          be used.  Otherwise nTestParm is an index into DataRates[] and
          the data rate is selected from there.

          While running display [W-X] on successful write, where "X" is
          replaced by the line number.
*/
void TestWrBisync(void)
{
   int nLn;                            /* line number */
   long DataRate;
   int i,j;

   /* Get the data rate from the table or from #define TWB_DATARATE */
   if(nTestParm < 0 || nTestParm >= NUM_DATARATES)
      nTestParm = 0;
   if(nTestParm == 0)
      DataRate = TWB_DATARATE;
   else
      DataRate = DataRates[nTestParm];

   /* Initialize output buffer for bisync */
   for(i = 1;i < TWB_WRSIZE;i += 80)
   {
      j = 80;
      if(i + 80 > TWB_WRSIZE)      /* this copy reaches end of buffer */
         j = TWB_WRSIZE - i;
      strncpy(&sOutBuf[i],s1,j);
   }
   sOutBuf[0] = SOH;                   /* framing characters */
   sOutBuf[TWB_WRSIZE - 1] = ETX;

   /* Open all lines */
   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      if(!hiOpenBisync(nLn,MSG_MDM_DTR | MSG_MDM_RTS,TWB_CLK,DataRate))
      {
         printf("Open Failure - Line %d\n",nLn);
         ExitTest();
      }
      LnInfo[nLn].nLnOpen = TRUE;
   }

   while(!kbhit())
   {
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(LnInfo[nLn].nTxBufCnt < TXBQ_SIZE) /* empty Tx buffers avail */
         {
            if(hiWriteSync(nLn,sOutBuf,TWB_WRSIZE)) /* successful write */
            {
               LnInfo[nLn].nTxBufCnt = hiGetTxBufCnt(nLn);
               if(LnInfo[nLn].nLoop1 >= PRINTCNT)
                  printf("[W-%d]",nLn);
               LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
            }
            else                       /* write failed */
            {
               printf("Write Failure - Line %d\n",nLn);
               ExitTest();
            }
         }
      }
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestRdBisync
Purpose:  Test hiReadSync() on BISYNC open on sequential lines.
Comments: Compile control program with #define LNX_BISYNC set to 1.

          This test requires that the Rx clock be DCE sourced.

          While running display [R-X-Y-Z] each time a frame is received, where
          X is the line number, Y is the number of bytes in the received
          frame, and Z is the first nTestParm bytes (in hex) in the
          received frame.

          Set 4th command line parameter (nTestParm) to the number of
          received bytes to display.
*/
void TestRdBisync(void)
{
   int nLn;                            /* line number */
   int nRx;                            /* number of received data bytes */
   BYTE_T *pBuf;                       /* ptr to buffer for Rx frame */
   int nDisplay;                       /* number of Rx bytes to display */
   int i;

   /* Open all lines */
   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      if(!hiOpenBisync(nLn,MSG_MDM_DTR | MSG_MDM_RTS,MSG_CLK_1,0))
      {
         printf("Open Failure - Line %d\n",nLn);
         ExitTest();
      }
      LnInfo[nLn].nLnOpen = TRUE;
   }

   while(!kbhit())
   {
      ProcessMsg();                    /* check for system messages */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(LnInfo[nLn].nRxBufCnt)     /* an Rx buffer is available */
         {
            pBuf = LnInfo[nLn].sRxBuf;
            nRx = hiReadSync(nLn,pBuf);
            LnInfo[nLn].nRxBufCnt = hiGetRxBufCnt(nLn); /* update Rx buf cnt */
            if(nRx)
            {
               if(LnInfo[nLn].nLoop1 >= PRINTCNT)
               {
                  if(nTestParm == 0)
                     printf("[R-%d-%d]",nLn,nRx);
                  else
                  {
                     if(nRx < nTestParm)
                        nDisplay = nRx;
                     else
                        nDisplay = nTestParm;
                     printf("[R-%d-%d-",nLn,nRx);
                     for(i = 0;i < nDisplay;i++)
                        printf(" %x",pBuf[i]);
                     printf("]\n");
                  }
               }
               LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
            }
         }
      }
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestWrRdBisync
Purpose:  Test hiWriteSync(), hiReadSync() on BISYNC open with random length
          writes/reads on random writes lines, sequential read lines.  The
          read will verify the line, sequence number, and length.
Comments: Compile control program with #define LNX_BISYNC set to 1.

          Loopback plugs must be installed connecting TX-RX and TxClk-RxClk.

          Set #define TWRS_WRSIZE to the max number of bytes to send on each
          write.

          Set #define TWRS_DATARATE to the data rate in BPS.  If the
          4th command line parameter (nTestParm) is set to 0 TWRS_DATARATE will
          be used.  Otherwise nTestParm is an index into DataRates[] and
          the data rate is selected from there.

          While running display [W-X] on successful write and [R-X] on
          successful read, where "X" is replaced by the line number.
*/
void TestWrRdBisync(void)
{
   int i,nLn;
   char *pBuf;
   long DataRate;
   int nTxCnt,nRxCnt;       /* Tx and Rx byte counts */
   int nRecLn;              /* received line number */
   int nRecSeq;             /* received sequence number */
   int nRecLen;             /* received string length */
   static char sTemp[22];
   char c;

   /* Get the data rate from the table or from #define TWH_DATARATE */
   if(nTestParm < 0 || nTestParm >= NUM_DATARATES)
      nTestParm = 0;
   if(nTestParm == 0)
      DataRate = TWH_DATARATE;
   else
      DataRate = DataRates[nTestParm];

   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      pBuf = LnInfo[nLn].sTxBuf;
      pBuf[0] = SOH;
      pBuf[1] = '0';
      strncpy(pBuf+2,s2,80);  /* output str has line num, seq num, & length*/
      strncpy(pBuf+82,sOutBuf,BUF_SIZE - 83);
      if(!hiOpenBisync(nLn,MSG_MDM_DTR | MSG_MDM_RTS,MSG_CLK_2,DataRate))
      {
         printf("Open Failure - Line %d\n",nLn);
         ExitTest();
      }
      LnInfo[nLn].nLnOpen = TRUE;
      LnInfo[nLn].nTxSeq = 0;
      LnInfo[nLn].nRxSeq = 0;
   }
   sTemp[20] = '\0';
   while(1)
   {
      if(kbhit())
      {
         getch();
         ExitTest();
      }

      /* Write data */
      ProcessMsg();                    /* check for system messages */
      nLn = GetRandom(nFirstLn,nLastLn); /* get random line number */
      if(LnInfo[nLn].nTxBufCnt < TXBQ_SIZE) /* controller Tx buffers avail */
      {
         pBuf = LnInfo[nLn].sTxBuf;    /* get ptr to system Tx buffer */
         nTxCnt = GetRandom(23,TWRS_WRSIZE); /* write size */
         sprintf(&pBuf[3],"%-3d",nLn); /* fill in string */
         sprintf(&pBuf[9],"%-3d",LnInfo[nLn].nTxSeq);
         sprintf(&pBuf[15],"%-4d",nTxCnt);
         for(i = 4;i <= 19;i++)      /* replace NULLs with printable char */
         {
            if(pBuf[i] == '\0')
               pBuf[i] = ' ';
         }
         c = pBuf[nTxCnt - 1];         /* save last char in Tx string */
         pBuf[nTxCnt - 1] = ETX;       /* last char becomes framing char */
         if(hiWriteSync(nLn,pBuf,nTxCnt) != 1)
         {
            printf("Write Failure - Line %d\n",nLn);
            ExitTest();
         }
         pBuf[nTxCnt - 1] = c;        /* restore last char in Tx string */
         LnInfo[nLn].nTxBufCnt = hiGetTxBufCnt(nLn);
         if(LnInfo[nLn].nLoop1 >= PRINTCNT)
            printf("[W-%d-%d-%d]",nLn,LnInfo[nLn].nTxSeq,nTxCnt);
         LnInfo[nLn].nLoop1 = (LnInfo[nLn].nLoop1 + 1) % (PRINTCNT + 1);
         LnInfo[nLn].nTxSeq = (LnInfo[nLn].nTxSeq + 1) % 1000;
      }

      /* Read data.  The 1st 20 bytes contain the line num, seq num, and
         length of the receive string. */
      ProcessMsg();                    /* check for system messages */
      for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
      {
         if(LnInfo[nLn].nRxBufCnt)     /* Rx buffers are available */
         {
            pBuf = LnInfo[nLn].sRxBuf; /* get ptr to Rx buffer */
            nRxCnt = hiReadSync(nLn,pBuf); /* read a frame of data */
            LnInfo[nLn].nRxBufCnt = hiGetRxBufCnt(nLn);
            if(nRxCnt > 0)             /* read successful */
            {
               if(nRxCnt < 20)         /* frame too short */
               {
                  printf("Rx frame too short - Line %d, Cnt %d\n",nLn,nRxCnt);
                  ExitTest();
               }
               strncpy(sTemp,pBuf,26); /* save 1st 25 bytes for error msg */
               for(i = 3;i <= 18;i++)  /* replace spaces with NULLs */
               {
                  if(pBuf[i] == ' ')
                  pBuf[i] = '\0';
               }
               nRecLn = atoi(&pBuf[2]); /* get message info */
               nRecSeq = atoi(&pBuf[8]);
               nRecLen = atoi(&pBuf[14]);
               if((nRecLn != nLn) ||   /* bad Rx line number */
                  (nRecSeq != LnInfo[nLn].nRxSeq) || /* bad Rx sequence number */
                  (nRecLen != nRxCnt+2) || /* bad Rx len (SOH & ETX stripped) */
                  (nRecLen > TWRA_WRSIZE))

               {
                  printf("\nRx Fail [nLn:%d:%d][Seq:%d:%d][Len:%d:%d][BufCnt:%d:%d]\n",
                         nLn,nRecLn,LnInfo[nLn].nRxSeq,nRecSeq,nRxCnt+2,nRecLen,
                         LnInfo[nLn].nRxBufCnt,hiGetRxBufCnt(nLn));
                  printf("Err Str:%s\n",sTemp); /* dump saved string */
                  printf("Err Str Hex:");
                  for(i = 0;i < 25;i++)
                     printf("%x ",sTemp[i]);
                  printf("\nCR to close and exit:  ");
                  while(!kbhit());
                  getch();
                  printf("\n");
                  ExitTest();
               }
               if(LnInfo[nLn].nLoop2 >= PRINTCNT)
                    printf("[R-%d-%d-%d]",nLn,LnInfo[nLn].nRxSeq,nRxCnt);
               LnInfo[nLn].nLoop2 = (LnInfo[nLn].nLoop2 + 1) % (PRINTCNT + 1);
               LnInfo[nLn].nRxSeq = (LnInfo[nLn].nRxSeq + 1) % 1000;
            }
            else
            {
               printf("Zero length read Failure - Line %d\n",nLn);
               ExitTest();
            }
         }
      } // end for(nLn)
   }
}

/*****************************************************************************
Function: TestSfg1
Purpose:  Test SFGs.
Comments: Compile control program with TESTSFG1 define set to 1.

          Should see a dot on console every few seconds.  Can also view SFGs
          in dual port memory with a debugger, should see SFG0 and 1 change
          between 0 & 1 occassionally, the remaining SFGs should be changing
          values continually.
*/
void TestSfg1(void)
{
   WORD_T i;                           /* SFG group number */
   UWORD_T Val;                        /* value of SFG */
   UWORD_T j;                          /* ripple counter */

   while(!kbhit())
   {
      /* Subtest 1, controller initialize - system check */
      hiSetSfg(0,1);                    /* signal system ready */
      while(!hiGetSfg(1,1));            /* wait for controller ready */
      hiClrSfg(1,1);                    /* clear controller ready */
      for(i = 2;i < SFG_SIZE;i++)      /* check each SFG */
      {
         if(hiGetSfg(i,0xffff) != 0xaa00 + i)
         {
            printf("Subtest1 failure\n");
            ExitTest();
         }
      }

      /* Subtest 2, controller clear - system check */
      hiSetSfg(0,1);                    /* signal system ready */
      while(!hiGetSfg(1,1));            /* wait for controller ready */
      hiClrSfg(1,1);                    /* clear controller ready */
      for(i = 2;i < SFG_SIZE;i++)      /* check each SFG */
      {
         if(hiGetSfg(i,0xffff) != 0)
         {
            printf("Subtest2 failure\n");
            ExitTest();
         }
      }

      /* Subtest 3, controller progressively set flag bits - system check */
      j = 1;                        /* progressively set & check flag bits */
      Val = 0;
      do
      {
         hiSetSfg(0,1);                  /* signal system ready */
         while(!hiGetSfg(1,1));          /* wait for controller ready */
         hiClrSfg(1,1);                  /* clear controller ready */
         for(i = 2;i < SFG_SIZE;i++)
         {
            Val |= j;
            if(hiGetSfg(i,0xffff) != Val)
            {
               printf("Subtest3 failure\n");
               ExitTest();
            }
         }
         j = j << 1;
      } while(j > 0);

      /* Subtest 4, controller progressively clear flag bits - system check */
      j = 0xfffe;
      do
      {
         hiSetSfg(0,1);                  /* signal system ready */
         while(!hiGetSfg(1,1));          /* wait for controller ready */
         hiClrSfg(1,1);                  /* clear controller ready */
         for(i = 2;i < SFG_SIZE;i++)
         {
            Val &= j;
            if(hiGetSfg(i,0xffff) != Val)
            {
               printf("Subtest4 failure\n");
               ExitTest();
            }
         }
         j = (j << 1) + 1;
      } while(j < 0xffff);

      /* Subtest 5, system initialize - controller check */
      for(i = 2;i < SFG_SIZE;i++)       /* initialize each SFG */
      {
         hiInitSfg(i,0xaa00 + i);
      }
      hiSetSfg(0,1);                     /* signal system ready */
      while(!hiGetSfg(1,1));             /* wait for controller ready */
      hiClrSfg(1,1);                     /* clear controller ready */

      /* Subtest 6, system clear - controller check */
      for(i = 2;i < SFG_SIZE;i++)      /* clear each SFG */
      {
         hiClrSfg(i,0xffff);
      }
      hiSetSfg(0,1);                     /* signal system ready */
      while(!hiGetSfg(1,1));             /* wait for controller ready */
      hiClrSfg(1,1);                     /* clear controller ready */

      /* Subtest 7, system progressively set flag bits - controller check */
      j = 1;
      do
      {
         for(i = 2;i < SFG_SIZE;i++)
         {
            hiSetSfg(i,j);              /* set flag */
         }
         j = j << 1;
         hiSetSfg(0,1);                     /* signal system ready */
         while(!hiGetSfg(1,1));             /* wait for controller ready */
         hiClrSfg(1,1);                     /* clear controller ready */
      } while(j > 0);

      /* Subtest 8, system progressively clear flag bits - controller check */
      j = 0xfffe;
      do
      {
         for(i = 2;i < SFG_SIZE;i++)
         {
            hiClrSfg(i,~j);
         }
         j = (j << 1) + 1;
         hiSetSfg(0,1);                     /* signal system ready */
         while(!hiGetSfg(1,1));             /* wait for controller ready */
         hiClrSfg(1,1);                     /* clear controller ready */
      } while(j < 0xffff);

      if(LnInfo[0].nLoop1 >= PRINTCNT)
         printf(".");
      LnInfo[0].nLoop1 = (LnInfo[0].nLoop1 + 1) % (PRINTCNT + 1);
   }
   getch();
   ExitTest();
}

/*****************************************************************************
Function: TestSfg2
Purpose:  Test TestSetSfg(), IncrementSfg(), DecrementSfg().
Comments: Compile control program with TESTSFG2 define set to 1.

          Uses SFG0 as semaphore guarding shared resource.  The shared
          resource is SFG1.  SFG2 and 3 are used to provide a visual
          indication of how often each side gets the resource, SFG2 is
          incremented each time the controller wins, and SFG3 each time
          the system wins. To see the visual indicators must view dual
          port memory with a debugger.

          SFG4 is decremented by the controller each time through the
          test loop just to exercise DecrementSfg().  SFG5 is decremented
          by the system each time through the test loop just to exercise
          DecrementSfg().

          Should see [W,X,Y,Z] print on screen every few seconds, with W being
          count of controller resource wins (SFG2), and X being count of
          application wins (SFG3).  These counts should be constantly
          increasing.  Y is controller side decrementer (SFG4) and Z is
          system side decrementer (SFG5).  These counts should be
          constantly decreasing.


*/
void TestSfg2(void)
{
   UWORD_T Val;                        // value of shared resource (SFG1)
   WORD_T i,j;
   UWORD_T w,x,y,z;                    // the W, X, Y, and Z display values

   while(!kbhit())
   {
      while(hiTestSetSfg(0,1));     // get semaphore
      x = hiIncrementSfg(3);        // bump system win count
      w = hiGetSfg(2,0xffff);       // get controller win count
      y = hiGetSfg(4,0xffff);       // get controller decrementer
      z = hiDecrementSfg(5);        // system decrementer
      Val = 0;
      hiInitSfg(1,Val);             // operate on shared resource
      while(Val < 0x40)             // operate on shared resource
      {
         hiIncrementSfg(1);
         Val++;
         for(j = 0;j < 5;j++)       // waste some time to allow controller to
         {                          //    change it if semaphore not working
            i = j;
            i = i;
         }
         if(hiGetSfg(1,0xffff) != Val) // shared resource collision
         {
            printf("Shared resource collision failure\n");
            ExitTest();
         }
      }
      hiClrSfg(0,1);             /* release semaphore */

      if(LnInfo[0].nLoop1 >= PRINTCNT)
         printf("[%x,%x,%x,%x]",w,x,y,z);
      LnInfo[0].nLoop1 = (LnInfo[0].nLoop1 + 1) % (PRINTCNT + 1);
   }
   getch();
   ExitTest();
}

/******************************************************************************
Function: TestAckAsync
Purpose:  Exercise the Windows 3.1 RocketPort driver Test25. Each time an
          ACK is received a 5 byte record is sent.
Comments: Compile control program with #define LNX_ASYNC set to 1.

          Cable FirstLn to the RocketPort port running Windows Test25.

          Windows Test25 must be started before this test is started, because
          this test initiates the handshake by sending the record.
*/
void TestAckAsync(void)
{
   int i, nLn;
   static char sTBuf[6];
   static char sRBuf[11];

   // Initialize output buffer, open line
   nLn = nFirstLn;
   strcpy(sTBuf,"12345");
   if(!hiOpenAsync(nLn,MSG_BAUD_9600,MSG_PAR_NONE,MSG_DATABIT_8,
                      MSG_STOPBIT_1,MSG_MDM_DTR | MSG_MDM_RTS))
   {
      printf("Open Failure - Line %d\n",nLn);
      ExitTest();
   }
   LnInfo[nLn].nLnOpen = TRUE;

   while(1)                            // Test loop
   {
      if(kbhit())
      {
         getch();
         ExitTest();
      }
      ProcessMsg();                    // check for system messages

      // Write the 5 byte record
      hiDelay(8000);                    // only send a few records a second
      i = hiWriteAsync(nLn,sTBuf,5);
      if(i != 5)
      {
         printf("  Write Error %d\n",i);
         ExitTest();
      }
      else
      {
         printf("W");
      }

      // Wait to receive the ACK
      while(1)
      {
         if(kbhit())
         {
            getch();
            ExitTest();
         }
         i = hiReadAsync(nLn,sRBuf,10);
         if(i == 0)
         {
            printf(".");
            hiDelay(100);
         }
         else if(i == 1)
         {
            printf("R");
            break;
         }
         else
         {
            printf("  Read Error %d",i);
            ExitTest();
         }
      }
   }
} /* end TestAckAsync() */

/*****************************************************************************
Function: GetRandom
Purpose:  Get a random number within the given range.
Call:     GetRandom(Start,End)
          int Start;  Start of random number range
          int End;  End of random number range
Return:   int: The random number
*/
int GetRandom(int Start,int End)
{
   int x;

   while(1)
   {
      x = rand() % (End + 1);
      if(x >= Start)
         break;
   }
   return(x);
}

/*******************************************************************************
Function: ProcessMsg
Purpose:  Process controller to system messages.
Call:     ProcessMsg
Return:   void
Comments: This function dequeues and displays all system messages.  Only the
          first 4 bytes of each message are displayed, since no system
          messages use more than 4 bytes.
*/
void ProcessMsg(void)
{
   static char Msg[MSG_LEN];

   while(SysMsgAvail)                  /* while int found sys messages */
   {
      SysMsgAvail = 0;
      while(hiDeqSysMsg(Msg))          /* while messages remain */
      {
         printf("\n[**MSG:%x %x %x %x %x %x]",Msg[0],Msg[1],Msg[2],Msg[3],
                Msg[4],Msg[5]);
      }
   }
}

/*****************************************************************************
Function: ExitTest
Purpose:  Close all open lines and exit the test.  Lets Tx buffers drain
          before closing each line.
Warnings: It is important that all lines be closed before exiting the program.
          This will prvent the controller from generating any more interrupts
          to the system after the system ISR is gone.
*/
void ExitTest(void)
{
   int nLn;

   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      if(LnInfo[nLn].nLnOpen)
      {
         hiClose(nLn);
         printf("[C:%d]",nLn);
      }
   }
   ProcessMsg();                       /* check for system messages */
   hiRestoreIrq();
   exit(0);
}

/*******************************************************************************
Function: IntHandler
Purpose:  Interrupt service routine for controller to system interrupts.
Call:     IntHandler()
Return:   void
Comments: The controller generates interrupts when it enqueues system messages,
          when it enqueues synchronous Rx buffers, and when it dequeues
          synchronous Tx buffers.  This handler updates system copies of the
          system message queue status, Rx and Tx synchronous buffer queue
          counts when the controller changes these counts.  By keeping system
          copies of these values frequent system access to dual port
          memory to check them is avoided, which would slow down the controller
          CPU.
Warnings: If using a Microsoft(R) compiler, make sure any functions called
          from IntHandler() have stack checking disabled.
*/
#ifdef MSC
#pragma check_stack( off )
#endif
void interrupt far IntHandler(void)
{
   int nLn;

   if(hiGetSysMsgStatus())
      SysMsgAvail = 1;
   for(nLn = nFirstLn;nLn <= nLastLn;nLn++)
   {
      LnInfo[nLn].nTxBufCnt = hiGetTxBufCnt(nLn);
      LnInfo[nLn].nRxBufCnt = hiGetRxBufCnt(nLn);
   }
   hiEndOfInterrupt();
}
#ifdef MSC
#pragma check_stack( on )
#endif











